import datetime
import os
import random
import shutil
import string
import unittest

from src.galleryinfo_parser.galleryinfo_parser import parse_galleryinfo


def random_string(length: int) -> str:
    return "".join(
        random.choice(string.ascii_letters + string.digits) for x in range(length)
    )


class TestGalleryInfoParser(unittest.TestCase):
    def _generate_galleryinfo(
        self, tags: str, withcomments: bool = False
    ) -> tuple[dict[str, str], str]:
        if tags == "":
            raise ValueError("tags cannot be empty")

        elements = dict[str, str]()
        elements["Title"] = random_string(10)
        elements["Upload Time"] = "2007-04-13 03:43"
        elements["Uploaded By"] = random_string(10)
        elements["Downloaded"] = "2007-08-13 03:43"
        elements["Tags"] = tags
        if withcomments:
            elements["Uploader's Comments"] = "\n" + random_string(10) + "\n"
        end = "Downloaded from E-Hentai Galleries by the Hentai@Home Downloader <3\n"

        return elements, "\n".join(
            ["\n".join([f"{key}: {value}" for key, value in elements.items()]), end]
        )

    def _assert_galleryinfo(
        self,
        gidpair: tuple[str, int],
        tagspair: tuple[str, list[tuple[str, str]]],
        withcomments: bool,
    ):
        """
        gidpair: (gallery_folder, gid)
        tagspair: (tags, tags_answer)
        """

        gallery_folder = gidpair[0]
        gid = gidpair[1]

        tags = tagspair[0]
        tags_answer = tagspair[1]

        dir_path = os.path.join(os.path.dirname(__file__), gallery_folder)
        if os.path.exists("dir_path"):
            raise FileExistsError(f"{dir_path} already exists")
        os.mkdir(dir_path)

        answer_element, txt = self._generate_galleryinfo(
            tags=tags, withcomments=withcomments
        )
        galleryinfo_path = os.path.join(dir_path, "galleryinfo.txt")
        with open(galleryinfo_path, "w") as file:
            file.write(txt)

        gallery = parse_galleryinfo(dir_path)
        shutil.rmtree(dir_path)

        self.assertEqual(gallery.gallery_name, gallery_folder)
        self.assertEqual(gallery.gid, gid)
        self.assertEqual(gallery.files_path, ["galleryinfo.txt"])
        self.assertEqual(
            gallery.modified_time, datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        )
        self.assertEqual(gallery.title, answer_element["Title"])
        self.assertEqual(gallery.upload_time, answer_element["Upload Time"])
        self.assertEqual(gallery.upload_account, answer_element["Uploaded By"])
        self.assertEqual(gallery.download_time, answer_element["Downloaded"])
        if withcomments:
            self.assertEqual(
                gallery.galleries_comments, answer_element["Uploader's Comments"][1:-1]
            )
        else:
            self.assertEqual(gallery.galleries_comments, "")
        self.assertEqual(gallery.tags, tags_answer)

    def test_gid(self):
        tagspair = ("tag1:value1", [("tag1", "value1")])
        self._assert_galleryinfo(("1", 1), tagspair, False)
        self._assert_galleryinfo(("[1] [3]", 3), tagspair, False)
        self._assert_galleryinfo(("1 [3]", 3), tagspair, False)
        self._assert_galleryinfo(("[1 [3]", 3), tagspair, False)
        self._assert_galleryinfo(("1] [3]", 3), tagspair, False)

    def test_tags(self):
        gidpair = ("1", 1)
        self._assert_galleryinfo(gidpair, ("tag1:value1", [("tag1", "value1")]), False)
        self._assert_galleryinfo(
            gidpair,
            ("tag1:value1, tag1:value2", [("tag1", "value1"), ("tag1", "value2")]),
            False,
        )
        self._assert_galleryinfo(gidpair, ("value1", [("untagged", "value1")]), False)
        self._assert_galleryinfo(
            gidpair,
            ("tag1:value1, tag2:value2", [("tag1", "value1"), ("tag2", "value2")]),
            False,
        )
        self._assert_galleryinfo(
            gidpair,
            ("tag1:value1, value2", [("tag1", "value1"), ("untagged", "value2")]),
            False,
        )

    def test_comments(self):
        gidpair = ("1", 1)
        tagspair = ("tag1:value1", [("tag1", "value1")])
        self._assert_galleryinfo(gidpair, tagspair, False)
        self._assert_galleryinfo(gidpair, tagspair, True)



__all__ = ["parse_gid", "parse_galleryinfo", "GalleryInfoParser"]


import datetime
import os


def count_files_in_directory(directory_path) -> int:
    return len(
        [
            f
            for f in os.listdir(directory_path)
            if os.path.isfile(os.path.join(directory_path, f))
        ]
    )


class GalleryInfoParser:
    """
    A class that represents a parser for gallery information.

    Attributes:
        gallery_name (str): The name of the gallery.
        gid (int): The gallery ID.
        files_path (list[str]): The paths of the files in the gallery.
        modified_time (datetime.datetime): The modified time of the gallery.
        title (str): The title of the gallery.
        upload_time (datetime.datetime): The upload time of the gallery.
        galleries_comments (str): The uploader's comment for the gallery.
        upload_account (str): The account used to upload the gallery.
        download_time (datetime.datetime): The download time of the gallery.
        tags (dict[str, str]): The tags associated with the gallery.
        pages (int): The number of pages in the gallery.
    """

    __slots__ = [
        "gallery_folder",
        "gallery_name",
        "gid",
        "files_path",
        "modified_time",
        "title",
        "upload_time",
        "galleries_comments",
        "upload_account",
        "download_time",
        "tags",
        "_pages",
    ]

    def __init__(
        self,
        gallery_folder: str,
        gallery_name: str,
        gid: int,
        files_path: list[str],
        modified_time: datetime.datetime,
        title: str,
        upload_time: datetime.datetime,
        galleries_comments: str,
        upload_account: str,
        download_time: datetime.datetime,
        tags: list[tuple[str, str]],
    ) -> None:
        self.gallery_folder = gallery_folder
        self.gallery_name = gallery_name
        self.gid = gid
        self.files_path = files_path
        self.modified_time = modified_time
        self.title = title
        self.upload_time = upload_time
        self.galleries_comments = galleries_comments
        self.upload_account = upload_account
        self.download_time = download_time
        self.tags = tags
        self._pages = -1

    @property
    def pages(self) -> int:
        if self._pages == -1:
            self._pages = count_files_in_directory(self.gallery_folder) - 1
        return self._pages

    def __repr__(self) -> str:
        return f"GalleryInfoParser(gallery_name={self.gallery_name}, gid={self.gid}, files_path={self.files_path}, modified_time={self.modified_time}, title={self.title}, upload_time={self.upload_time}, galleries_comments={self.galleries_comments}, upload_account={self.upload_account}, download_time={self.download_time}, tags={self.tags})"

    def __str__(self) -> str:
        return self.__repr__()


def parse_gid(gallery_folder: str) -> int:
    """
    Parses the gallery ID from the given folder path.

    Args:
        gallery_folder (str): The path to the folder containing the gallery information.

    Returns:
        int: The gallery ID.
    """
    gallery_name = os.path.basename(gallery_folder)
    if "[" in gallery_name and "]" in gallery_name:
        gid = int(gallery_name.split("[")[-1].replace("]", ""))
    else:
        gid = int(gallery_name)
    return gid


def parse_galleryinfo(gallery_folder: str) -> GalleryInfoParser:
    """
    Parses the gallery information from the given folder path.

    Args:
        gallery_folder (str): The path to the folder containing the gallery information.

    Returns:
        GalleryInfoParser: An instance of the GalleryInfoParser class containing the parsed gallery information.
    """
    gallery_info_path = os.path.join(gallery_folder, "galleryinfo.txt")
    with open(gallery_info_path, "r", encoding="utf-8") as file:
        lines = file.read().strip("\n").split("\n")

    gallery_name = os.path.basename(gallery_folder)
    gid = parse_gid(gallery_folder)
    files_path = os.listdir(gallery_folder)
    modified_time = datetime.datetime.fromtimestamp(os.path.getmtime(gallery_info_path))

    comments = False
    comment_lines = list()
    for line in lines:
        if "Uploader's Comments" in line:
            comments = True
        elif comments:
            if (
                line
                == "Downloaded from E-Hentai Galleries by the Hentai@Home Downloader <3"
            ):
                break
            comment_lines.append(line.strip())
        elif ":" in line:
            key, value = line.split(":", 1)
            key = key.strip()
            value = value.strip()
            match key:
                case "Tags":
                    tags = list[tuple[str, str]]()
                    for tag in value.split(","):
                        if ":" in tag:
                            tag_key, tag_value = tag.split(":", 1)
                            if tag_key.strip() != "":
                                tags.append((tag_key.strip(), tag_value.strip()))
                            else:
                                tags.append(("untagged", tag_value.strip()))
                        else:
                            tags.append(("untagged", tag.strip()))
                case "Title":
                    title = value
                case "Upload Time":
                    upload_time = datetime.datetime.strptime(value, "%Y-%m-%d %H:%M")
                case "Uploaded By":
                    upload_account = value
                case "Downloaded":
                    download_time = datetime.datetime.strptime(value, "%Y-%m-%d %H:%M")

    galleries_comments = "\n".join(comment_lines).strip("\n")

    return GalleryInfoParser(
        gallery_folder=gallery_folder,
        gallery_name=gallery_name,
        gid=gid,
        files_path=files_path,
        modified_time=modified_time,
        title=title,
        upload_time=upload_time,
        galleries_comments=galleries_comments,
        upload_account=upload_account,
        download_time=download_time,
        tags=tags,
    )



__all__ = ["GalleryURLParser"]


import re


class GalleryURLParser:
    """
    A parser class for extracting gallery information from URLs.
    Attributes:
        gid (int): The gallery ID.
        url (str): The gallery URL.
        url_key (str): The key associated with the gallery URL.
    Methods:
        __init__(): Initializes the GalleryURLParser with default values.
        gid: Property to get or set the gallery ID.
        url: Property to get or set the gallery URL. The URL must be from exhentai.org or e-hentai.org.
    Examples:
        >>> parser = GalleryURLParser()
        >>> parser.url = "https://exhentai.org/g/1234567/abcdefg"
        >>> parser.gid
        1234567
        >>> parser.url_key
        "abcdefg"
    """

    __slots__ = ["gid", "url", "url_key"]

    def __init__(self) -> None:
        self.gid = 0
        self.url = ""
        self.url_key = ""

    @property
    def gid(self) -> int:
        if self.gid == 0:
            raise ValueError("GID is not set.")
        return self.gid

    @gid.setter
    def gid(self, gid: int) -> None:
        self.gid = gid

    @property
    def url(self) -> str:
        return self.url

    @url.setter
    def url(self, url: str) -> None:
        if url != "":
            if ("exhentai.org" not in self.url) and ("e-hentai.org" not in self.url):
                raise ValueError("The url is not the gallery's url.")
            self.url = url

            if "exhentai.org" in self.url:
                match = re.search(
                    r"https://exhentai\.org/g/(\d+)/([a-zA-Z0-9]+)", self.url
                )
            if "e-hentai.org" in self.url:
                match = re.search(
                    r"https://e-hentai\.org/g/(\d+)/([a-zA-Z0-9]+)", self.url
                )
            if match:
                self.gid = int(match.group(1))
                self.url_key = match.group(2)
            else:
                raise ValueError("The url is not the gallery's url.")
        else:
            raise ValueError("The url cannot be empty.")



__all__ = [
    "__version__",
    "__version_info__",
    "parse_gid",
    "parse_galleryinfo",
    "GalleryInfoParser",
    "GalleryURLParser",
]


from .galleryinfo_parser import parse_gid, parse_galleryinfo, GalleryInfoParser
from .gallery_url_parser import GalleryURLParser
